@@ -24,7 +24,7 @@ use swc_common::{
     errors::Handler,
     FileName, Mark, SourceMap,
 };
-use swc_ecma_ast::{EsVersion, Expr};
+use swc_ecma_ast::{EsVersion, Expr, Program};
 use swc_ecma_ext_transforms::jest;
 use swc_ecma_loader::resolvers::{
     lru::CachingResolver, node::NodeModulesResolver, tsc::TsConfigResolver,
@@ -180,18 +180,23 @@ impl Default for InputSourceMap {
 }
 
 impl Options {
-    pub fn build<'a>(
+    /// `parss`: `(syntax, target, is_module)`
+    pub fn build_as_input<'a, P>(
         &self,
         cm: &Arc<SourceMap>,
         base: &FileName,
+        parse: impl FnOnce(Syntax, EsVersion, bool) -> Result<Program, Error>,
         output_path: Option<&Path>,
         source_file_name: Option<String>,
         handler: &Handler,
         is_module: bool,
         config: Option<Config>,
         comments: Option<&'a SwcComments>,
-        custom_before_pass: impl 'a + swc_ecma_visit::Fold,
-    ) -> BuiltConfig<impl 'a + swc_ecma_visit::Fold> {
+        custom_before_pass: impl FnOnce(&Program) -> P,
+    ) -> Result<BuiltInput<impl 'a + swc_ecma_visit::Fold>, Error>
+    where
+        P: 'a + swc_ecma_visit::Fold,
+    {
         let mut config = config.unwrap_or_else(Default::default);
         config.merge(&self.config);
 
@@ -214,6 +219,8 @@ impl Options {
         let target = target.unwrap_or_default();
 
         let syntax = syntax.unwrap_or_default();
+
+        let program = parse(syntax, target, is_module)?;
         let mut transform = transform.unwrap_or_default();
 
         let regenerator = transform.regenerator.clone();
@@ -310,7 +317,7 @@ impl Options {
                 syntax.typescript()
             ),
             resolver_with_mark(top_level_mark),
-            custom_before_pass,
+            custom_before_pass(&program),
             // handle jsx
             Optional::new(
                 react::react(
@@ -325,7 +332,8 @@ impl Options {
             Optional::new(jest::jest(), transform.hidden.jest)
         );
 
-        BuiltConfig {
+        Ok(BuiltInput {
+            program,
             minify: config.minify,
             pass,
             external_helpers,
@@ -338,7 +346,7 @@ impl Options {
             output_path: output_path.map(|v| v.to_path_buf()),
             source_file_name,
             preserve_comments,
-        }
+        })
     }
 }
 
@@ -792,7 +800,8 @@ impl Config {
 }
 
 /// One `BuiltConfig` per a directory with swcrc
-pub struct BuiltConfig<P: swc_ecma_visit::Fold> {
+pub struct BuiltInput<P: swc_ecma_visit::Fold> {
+    pub program: Program,
     pub pass: P,
     pub syntax: Syntax,
     pub target: EsVersion,
